﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using OSGeo.FDO.Schema;
using OSGeo.FDO.Connections;
using OSGeo.FDO.ClientServices;
using OSGeo.FDO.Commands.DataStore;
using System.Diagnostics;
using System.IO;
using OSGeo.FDO.Commands.Schema;

namespace TestApp
{
    class Program
    {
        const string FILENAME = "Trac739.sqlite";

        static void Main(string[] args)
        {
            if (File.Exists(FILENAME))
                File.Delete(FILENAME);

            IConnection conn = FeatureAccessManager.GetConnectionManager().CreateConnection("OSGeo.SQLite");
            ICreateDataStore create = (ICreateDataStore)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_CreateDataStore);

            var dsprops = create.DataStoreProperties;
            dsprops.SetProperty("File", FILENAME);

            create.Execute();

            Debug.Assert(File.Exists(FILENAME));

            conn.ConnectionString = "File=" + FILENAME;
            var state = conn.Open();
            Debug.Assert(state == ConnectionState.ConnectionState_Open);

            IApplySchema apply = (IApplySchema)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_ApplySchema);
            apply.FeatureSchema = CreateTestSchema();
            apply.IgnoreStates = true;
            apply.Execute();


            IDescribeSchema describe = (IDescribeSchema)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_DescribeSchema);
            FeatureSchemaCollection schemas = describe.Execute();

            Debug.Assert(schemas.Count == 1);
            FeatureSchema schema = schemas[0];

            int idx = schema.Classes.IndexOf("DaKlass");
            Debug.Assert(idx >= 0);

            ClassDefinition cls = schema.Classes[idx];
            var props = cls.Properties;
            var idprops = cls.IdentityProperties;

            Debug.Assert(props.Count == 2);
            Debug.Assert(idprops.Count == 1);

            DataPropertyDefinition id = idprops[0];

            //Fails here.
            Debug.Assert(id.DataType == DataType.DataType_Int64, "Expected int64");
            Debug.Assert(id.IsAutoGenerated == false, "Expected IsAutoGenerated = false");
        }

        static FeatureSchema CreateTestSchema()
        {
            FeatureSchema schema = new FeatureSchema("Default", "Test Schema");

            FeatureClass fc = new FeatureClass("DaKlass", "");

            DataPropertyDefinition id = new DataPropertyDefinition("ID", "")
            {
                DataType = DataType.DataType_Int64,
                IsAutoGenerated = false
            };

            GeometricPropertyDefinition geom = new GeometricPropertyDefinition("Geom", "")
            {
                GeometryTypes = (int)GeometricType.GeometricType_All,
                HasElevation = false,
                HasMeasure = false
            };

            var props = fc.Properties;
            var idProps = fc.IdentityProperties;

            props.Add(id);
            props.Add(geom);

            idProps.Add(id);
            fc.GeometryProperty = geom;

            var classes = schema.Classes;

            classes.Add(fc);

            return schema;
        }
    }
}
